# -*- coding: utf-8 -*-
import os
import re
import subprocess
import sys

from . import vplanet_core as core
from .output import get_output


class VPLANETError(RuntimeError):
    """
    Catch-all runtime error class for VPLANET.

    """

    pass


def _entry_point():
    """
    ``vplanet`` command line script entry point.

    """
    return core.run(*sys.argv)


def run(
    infile="vpl.in", verbose=False, quiet=False, clobber=False, units=True, C=False
):
    """
    Run `vplanet` and return the output.

    Args:
        infile (str, optional): The path to the input file. Default ``vpl.in``.
        verbose (bool, optional): Enable verbose output? Default False.
        quiet (bool, optional): Suppress all output? Default False.
        clobber (bool, optional): Run ``vplanet`` even if a log file exists?
            Default False.
        units (bool, optional): If True, returns unit-ful output. If False, the
            output arrays are standard ``numpy`` arrays. Default True.

    Returns:
        A ``vplanet.Output`` object containing the full output from the run.

    Raises:
        ``vplanet.VPLANETError``: If something goes wrong in the C extension.

    .. note::

        We need to change the way vplanet exits on error in order to
        return to the Python interpreter when it terminates. The current
        hack is to spawn a subprocess so we don't terminate the current
        Python session.

    """
    # Determine the system name from the infile
    sysname = None
    with open(infile, "r") as f:
        lines = f.readlines()
        for line in lines:
            match = re.match("sSystemName[ \t\n]+(.*?)[ \t\n#]", line)
            if match:
                if len(match.groups()):
                    sysname = match.groups()[0]
                    break

    # Does the log file exist?
    path = os.path.abspath(os.path.dirname(infile))
    log_exists = os.path.exists(os.path.join(path, "{}.log".format(sysname)))

    # Run vplanet
    if clobber or not log_exists:

        # Parse kwargs
        if C:
            exe = "../../bin/vplanet"
        else:
            exe = "vplanet"
        args = [exe, infile]
        if verbose:
            args += ["-v"]
        if quiet:
            args += ["-q"]

        # Spawn `vplanet` as a subprocess
        error = False
        try:
            subprocess.check_output(args, cwd=path)
        except subprocess.CalledProcessError as e:
            error = True
        if error:
            raise VPLANETError("Error running VPLANET.")

    # Grab the output
    output = get_output(path=path, sysname=sysname, units=units)

    # We're done!
    return output


def help(verbose=False):
    from .vplanet_help import VPLANETHelp

    return VPLANETHelp(verbose=verbose)



# -*- coding: utf-8 -*-

# Grab version from the C extension
from .vplanet_core import version

__version__ = version().decode("utf-8")

# Grab the help info


# Import the rest of the user-facing stuff
from .log import Log, LogBody, LogStage

# Import the logger
from .logger import logger
from .output import Body, Output, get_output
from .quantity import VPLANETQuantity as Quantity

# Import the main interface
from .wrapper import VPLANETError, help, run



# -*- coding: utf-8 -*-
from astropy import units as u

# import Angle just so we have a more or less complete list of Quantity
# subclasses loaded - matplotlib needs them all separately!
# NOTE: in matplotlib >=3.2, subclasses will be recognized automatically,
# and once that becomes our minimum version, we can remove this,
# adding just u.Quantity itself to the registry.
from astropy.coordinates import Angle  # noqa
from matplotlib import units


# Get all subclass for Quantity, since matplotlib checks on class, not subclass.
def all_issubclass(cls):
    return {cls}.union([s for c in cls.__subclasses__() for s in all_issubclass(c)])


class quantity_support(units.ConversionInterface):

    _all_issubclass_quantity = all_issubclass(u.Quantity)

    def __init__(self):

        # Keep track of original converter in case the context manager is
        # used in a nested way.
        self._original_converter = {}

        for cls in self._all_issubclass_quantity:
            self._original_converter[cls] = units.registry.get(cls)
            units.registry[cls] = self

    @staticmethod
    def axisinfo(unit, axis):
        # We set the axes labels in `VPLANETFigure` instead
        return None

    @staticmethod
    def convert(val, unit, axis):
        if isinstance(val, u.Quantity):
            return val.to_value(unit)
        elif hasattr(val, "__len__") and len(val) == 0:
            return []
        elif isinstance(val, list) and isinstance(val[0], u.Quantity):
            return [v.to_value(unit) for v in val]
        else:
            return val

    @staticmethod
    def default_units(x, axis):
        if hasattr(x, "unit"):
            return x.unit
        return None

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        for cls in self._all_issubclass_quantity:
            if self._original_converter[cls] is None:
                del units.registry[cls]
            else:
                units.registry[cls] = self._original_converter[cls]



# -*- coding: utf-8 -*-
import astropy.units as u

# Aliases
custom_units = [
    #
    # Unitful quantities:
    #
    u.def_unit("TO", 1.39e21 * u.kg),  # Terrestrial ocean
    u.def_unit("LSUN", 3.846e26 * u.W),  # Solar luminosity
    #
    # Aliases that need to be standardized in VPLANET:
    #
    u.def_unit("sec", u.second),
    u.def_unit("years", u.year),
    u.def_unit("days", u.day),
    u.def_unit("bars", u.bar),
    u.def_unit("ergs", u.erg),
    u.def_unit("Earth Radii", u.Rearth),
    u.def_unit("Earth radii", u.Rearth),
    u.def_unit("Earthradii", u.Rearth),
    u.def_unit("Earth Masses", u.Mearth),
    u.def_unit("Degrees", u.deg),
    #
    # Non-standard quantities we'll interpret as unitless:
    #
    u.def_unit("F/F_Earth", u.Unit("")),
    u.def_unit("Initial Primordial Earth Number", u.Unit("")),
    u.def_unit("EMAGMOM", u.Unit("")),
    u.def_unit("EMAGPAUSERAD", u.Unit("")),
]

# Register the units
u.add_enabled_units(custom_units)

# Make them available as regular units
for unit in custom_units:
    name = unit.name.replace("/", "_").replace(" ", "")
    setattr(u, name, unit)
